-- Filename:     pwm_8bit.vhd
-- Filetype:     VHDL code (behavior)
-- Date:         31 january 2012
-- Update:       -
-- Description:  A behavioral description of a PWM system
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.0.1
-- Copyright:    (c)2012, De Haagse Hogeschool
--
-- This is an example of a behavior description of an
-- implementation of a digital PWM-system. For info
-- on PWM, start with 
-- http://en.wikipedia.org/wiki/Pulse-width_modulation

-- We need std_logic et al.
-- We need the numeric_std for the type unsigned
-- some operators ('+' and '>')

library ieee;
use ieee.std_logic_1164.all;
--use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

-- The entity
entity pwm_8bit is
    port (clk     : in std_logic;
          areset  : in std_logic;
          load    : in std_logic;
          data    : in unsigned (7 downto 0);
          top     : out std_logic;
          pwm_out : out std_logic
         );
end pwm_8bit;

-- The architecture
architecture rtl of pwm_8bit is
-- The internal counter keeps track of the PWM cycle
signal counter : unsigned (7 downto 0);
-- Internal storage for reference value
signal reg : unsigned (7 downto 0);
begin

    -- The conversion process is under control of a positive edge
	 -- of the clock, the asynchronous reset, and also the register
	 -- and the counter. The last two are used in comparisions and
	 -- are not under control of a clock signal (combinational logic).
	 process (areset, clk, reg, counter) is
	 begin
        -- There is an asynchronous reset
        if (areset = '1') then
            -- Reset both counter and reference value (reg)
            counter <= "00000000";
				reg <= "00000000";
        -- There is a positive edge				
        elsif rising_edge(clk) then
            -- Load is active
            if load = '1' then
               -- Load new reference value and restart PWM cycle
					reg <= data;
					counter <= "00000000";
				else
               -- Update PWM counter (continue the PWM cycle)
					counter <= counter + 1;
				end if;
        end if;

        -- Make a pulse if counter is at top, a nice feature to
		  -- synchronize external hardware to the PWM cycle
        if counter = "11111111" then
            top <= '1';
        else
            top <= '0';
        end if;

        -- PWM output is high if reference value is greater than counter
		  if reg > counter then
            pwm_out <= '1';
        else
            pwm_out <= '0';
        end if;

	 end process;
	 
end rtl;