# Filename:     tb_double_dabble_8bit.do
# Filetype:     Modelsim Script File
# Date:         24 maart 2011
# Update:       -
# Description:  Script File For Automatic Simulation
# Author:       J. op den Brouw
# State:        Demo
# Error:        -
# Version:      1.0
# Copyright:    (c)2011, De Haagse Hogeschool

# Transcript window
transcript on

# Recreate the RTL work directory
if {[file exists rtl_work]} {
	vdel -lib rtl_work -all
}
vlib rtl_work
vmap work rtl_work

# Find out if we're started through Quartus or by hand
# (or by using an exec in the Tcl window in Quartus).
# Quartus has the annoying property that it will start
# Modelsim from a directory called "simulation/modelsim".
# The design and the testbench are located in the project
# root, so we've to compensate for that.
if [ string match "*simulation/modelsim" [pwd] ] { 
	set prefix "../../"
	puts "Running Modelsim from Quartus..."
} else {
	set prefix ""
	puts "Running Modelsim..."
}

# Compile the VHDL description and testbench,
# please note that the design and its testbench are located
# in the project root, but the simulator start in directory
# <project_root>/simulation/modelsim, so we have to compensate
# for that.
vcom -2008 -work work ${prefix}pwm_8bit.vhd
vcom -2008 -work work ${prefix}tb_pwm_8bit.vhd

# Start simulation with 1 ns time step
vsim -t 1ns -L rtl_work -L work -voptargs="+acc" tb_pwm_8bit

# Log all signals in the design, good if the number
# of signals is small.
add log -r *

# Add all toplevel signals
# Add a number of signals of the simulated design
add list *
add list dut/counter
add list -radix hex dut/counter

# Add all toplevel signals
# Add a number of signals of the simulated design
add wave -divider "Inputs"
add wave areset
add wave clk
add wave load
add wave -radix hex data
add wave -divider "Internals"
#add wave dut/reg
add wave -radix hex dut/reg
#add wave dut/counter
add wave -radix hex dut/counter
# Add the counter as an analog signal to the Wave window
add wave -format analog-step -radix unsigned -min 0 -max 255 -height 100 dut/counter
add wave -divider "Outputs"
add wave top
add wave pwm_out

# Open Structure, Signals (waveform) and List window
view structure
view list
view signals
view wave

# Run simulation for 160000 ns
run 160000 ns

# Fill up the waveform in the window
wave zoom full

