-- Filename:     tb_pwm_8bit.vhd
-- Filetype:     VHDL testbench
-- Date:         7 march 2012
-- Update:       11 mar 2013
-- Description:  A testbench for a PWM system
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2013, De Haagse Hogeschool
--
-- This is an example of a testbench for a digital
-- PWM-system. For info on PWM, start with 
-- http://en.wikipedia.org/wiki/Pulse-width_modulation

-- We need std_logic et al.
-- We need numeric_std for the type unsigned.
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- The testbench has an empty entity
entity tb_pwm_8bit is
end entity tb_pwm_8bit;

-- The testbench
architecture testbench of tb_pwm_8bit is

-- Signals to monitor in the simulator
signal areset : std_logic;
signal clk : std_logic;
signal data : unsigned(7 DOWNTO 0);
signal load : std_logic;
signal pwm_out : std_logic;
signal top : std_logic;

-- Declaration of the Device Under Test as a component
component pwm_8bit
	port (
	areset : in std_logic;
	clk : in std_logic;
	data : in unsigned(7 downto 0);
	load : in std_logic;
	pwm_out : out std_logic;
	top : out std_logic
	);
end component;

begin

	-- Instantiating the Device Under Test
	dut : pwm_8bit
	port map ( areset => areset, clk => clk, data => data, load => load, pwm_out => pwm_out, top => top );
	
	-- Generating the clock signal at 5 MHz
	genclk : process
	begin
		loop
			clk <= '0';
			wait for 100 ns;
			clk <= '1';
			wait for 100 ns;
		end loop;
	end process genclk;

	-- Generating the data
	genstim : process
	begin
		-- Reset the system
		areset <= '1';
		load <= '0';
		wait for 50 ns;
		areset <= '0';
		wait until clk = '1';
		wait for 20 ns;
		
		-- Set the reference to 80H
		-- and run one PWM cycle
		data <= "10000000";
		load <= '1';
		wait until clk = '1';
		wait for 20 ns;
		load <= '0';

		for i in 0 to 254 loop
			wait until clk = '1';
		end loop;
		wait for 20 ns;

		-- Set the reference to 00H
		-- and run one PWM cycle
		data <= "00000000";
		load <= '1';
		wait until clk = '1';
		wait for 20 ns;
		load <= '0';

		for i in 0 to 254 loop
			wait until clk = '1';
		end loop;
		wait for 20 ns;

		-- Set the reference to FFH
		-- and run one PWM cycle
		data <= "11111111";
		load <= '1';
		wait until clk = '1';
		wait for 20 ns;
		load <= '0';

		for i in 0 to 255 loop
			wait until clk = '1';
		end loop;

		-- Stop simulation
		wait;
	end process genstim;

end testbench;
